
# ------------------------------------------------------------ #
# --- Plot imputation performance in single graph          --- #
# ------------------------------------------------------------ #

# --------------------------------------- #
# - Color palettes                      - #
# --------------------------------------- #

rm (list=ls())

Get.LD.Color <- function (value=NA, color.values=NA, color.steps=NA) {
  color <- color.values [length(color.values)]
  if (!is.na(value)) { color <-  ( color.values [ color.steps <= value ] )[1] }
  color
 }

LD.color.number    <- 5000
LD.color.sw.number <- 1000

# Color #
ld.blue2red.colors <- function (n) {
  if ((n <- as.integer(n[1])) > 0) { c( rainbow (n, start = 0, end = 4/6) ) }
    else                           { character(0)                           }
 }

# Black/white #
ld.white2black.colors <- function (n) {
  if ((n <- as.integer(n[1])) > 0) { gray ( (0:(n-1) ) / n) }
    else                           { character(0)                           }
 }


LD.colors        <- ld.blue2red.colors (LD.color.number)
LD.sw.colors     <- ld.white2black.colors (LD.color.sw.number)


#LD.colors       <- heat.colors    (LD.color.number)
#LD.colors       <- terrain.colors (LD.color.number)
#LD.colors       <- topo.colors    (LD.color.number)
#LD.colors       <- cm.colors      (LD.color.number)

LD.color.steps     <- 1 - 1/(LD.color.number   -1) * 0:(LD.color.number   -1)
LD.color.sw.steps  <- 1 - 1/(LD.color.sw.number-1) * 0:(LD.color.sw.number-1)

# --------------------------------------- #
# --------------------------------------- #



# --------------------------------------- #
# - Plotting functions                  - #
# --------------------------------------- #


  GetImputeAccuName <- function (array1=NA, array2=NA, method=NA, subdir="Concordance_Proportion_Data_noMAF") {
    midfix     <- "concordanceWith"
    suffix     <- "_MAFall"

    paste ( "./", subdir, "/", method, "/",  array1, midfix, array2, suffix, sep="")
  }
  
  
  GetImputePropName <- function (array1=NA, array2=NA, method=NA, subdir="Concordance_Proportion_Data_noMAF") {
    midfix     <- "Without"
    suffix     <- "_SNPcompared_ForPlots.txt"

#    paste ( "./", subdir, "/", method, "/",  array1, midfix, array2, suffix, sep="")
    paste ( "./", subdir, "/", method, "/",  array2, midfix, array1, suffix, sep="")
  }
  
    GetImputeAccuName ("Affy5", "Affy6", "MACH")
    GetImputePropName ("Affy5", "Affy6", "MACH")
  

  GetAccuRate <- function (array1=NA, array2=NA, method=NA) {
    read.table ( GetImputeAccuName (array1, array2, method), as.is=T, header=T )
   }
  
  GetPropRate <- function (array1=NA, array2=NA, method=NA) {
    read.table ( GetImputePropName (array1, array2, method), as.is=T, header=T )
   }

   GetAccuRate ("Affy5", "Affy6", "MACH")
   GetPropRate ("Affy5", "Affy6", "MACH")
   
   GetAccuRate ("Affy5", "Affy6", "BEAGLE")
   GetPropRate ("Affy5", "Affy6", "BEAGLE")

   
  GetAccuRates <- function (array1=NA, array2=NA) {
  
    b.accu <- GetAccuRate (array1, array2, "BEAGLE")
    i.accu <- GetAccuRate (array1, array2, "IMPUTE")
    m.accu <- GetAccuRate (array1, array2, "MACH")
    p.accu <- GetAccuRate (array1, array2, "PLINK")

    accu.rates <- cbind ( i.accu[,1], b.accu[,2], i.accu[,2], m.accu[,2], p.accu[,2] )
      colnames(accu.rates) <- c("Threshold", "BEAGLE", "IMPUTE", "MACH", "PLINK")

    accu.rates
   }

  GetPropRates <- function (array1=NA, array2=NA) {
  
    b.prop <- GetPropRate (array1, array2, "BEAGLE")
    i.prop <- GetPropRate (array1, array2, "IMPUTE")
    m.prop <- GetPropRate (array1, array2, "MACH")
    p.prop <- GetPropRate (array1, array2, "PLINK")

    prop.rates <- cbind ( i.prop[,2], b.prop[,3]/b.prop[,4], i.prop[,3]/i.prop[,4], m.prop[,3]/m.prop[,4], p.prop[,3]/p.prop[,4] )
      colnames(prop.rates) <- c("Threshold", "BEAGLE", "IMPUTE", "MACH", "PLINK")

    prop.rates
   }

    GetAccuRates("Affy5", "Affy6")
    GetPropRates("Affy5", "Affy6")


  ReScale <- function (values=NA, range=NA, size=NA) {
    (values-range[1])/(range[2]-range[1]) * size
   }


  # Accuray vs. Efficacy
  PlotAccuEffiCurve <- function (array1=NA, array2=NA, xy.origin=NA, accu.range=c(0.7,1.0), prop.range=c(0.0,1.0), x.default=NA,
                                 methods=NA, colors=1, graph.offset=NA) {

    accu.rates <- GetAccuRates(array1, array2)
    prop.rates <- GetPropRates(array1, array2)

    th   <- accu.rates[,"Threshold"]  ;  th   <- ReScale ( th  ,      c(0,1), 1.0-2.0*graph.offset )  # rescale to zoomed area
    accu <- accu.rates[,method.order] ;  accu <- ReScale ( accu, accu.range , 1.0-2.0*graph.offset )  # rescale to zoomed area
    prop <- prop.rates[,method.order] ;  prop <- ReScale ( prop, prop.range , 1.0-2.0*graph.offset )  # rescale to zoomed area

    for (i in 1:(dim(accu)[2])) {
      lines  ( xy.origin[1]+graph.offset+accu[,i], xy.origin[2]+graph.offset+prop[,i], lty=1 , col=colors[i] )
      points ( xy.origin[1]+graph.offset+accu[,i], xy.origin[2]+graph.offset+prop[,i], pch=20, col=colors[i] )
      th.default <- accu.rates[,"Threshold"] == x.default[i] ;  th.default[is.na(th.default)] <- F
      if (any(th.default)) {
        points ( xy.origin[1]+graph.offset+accu[th.default,i], xy.origin[2]+graph.offset+prop[th.default,i], pch=23, col=colors[i], cex=1.5 )
       }
     }

   }



  PlotCurves <- function (array1=NA, array2=NA, xy.origin=NA, 
                          accu.range=c(0.7,1.0), prop.range=c(0.0,1.0), x.default=NA,
                          methods=NA, colors=1, graph.offset=NA) {

    PlotAccuEffiCurve (array1=array1, array2=array2, xy.origin=xy.origin, x.default=x.default,
                       accu.range=accu.range, prop.range=prop.range, 
                       methods=methods, colors=colors, graph.offset=graph.offset) 
                   
  }
   
   
   
# --------------------------------------- #
# --------------------------------------- #





# --------------------------------------- #
# - Imputation plots                    - #
# --------------------------------------- #

  xlimits <- c(-0.6,3.0)
  ylimits <- c(-0.3,3.3)

  graph.offset <- 0.03
  
  platforms <- c("Affymetrix 5.0", "Affymetrix 6.0", "Illumina 550k")
  
  subdirname <- "Concordance_Proportion_Data_noMAF"
  programs   <- c("PLINK", "MACH", "IMPUTE", "BEAGLE")
  
  method.order    <- c("PLINK", "MACH", "IMPUTE", "BEAGLE")
  method.colors   <- c(4,2,3,6)
  method.defaults <- c(0.95, 0.30, 0.90, 0.40)


  
  # --- Imputation using CEU --- #
  

  graph.file     <- "accuracy_vs_efficacy.pdf"
  graph.title    <- NA
  graph.subtitle <- NA
  
  pdf        (file=graph.file, paper="special", height=4*(xlimits[2]-xlimits[1]), width=4*(xlimits[2]-xlimits[1]))
  
    plot (0,0, type="n", bty="n",
          xlim=xlimits, ylim=ylimits,
          xaxt="n", yaxt="n", xlab="", ylab="",
          main=graph.title  , cex.main=2.0,
          sub=graph.subtitle, cex.sub =1.3)

      # - Legends - #
      # x: Accuracy 
      for (i in 0:2) {
        lines  (x=i+c(0+graph.offset,1-graph.offset), y=rep(0,2)                )
          lines(x=i+rep(0.0+graph.offset,2)           , y=0+c(0,-graph.offset/2)  )
#          lines(x=i+rep(0.5             ,2)           , y=0+c(0,-graph.offset/2)  )
          lines(x=i+rep(1.0-graph.offset,2)           , y=0+c(0,-graph.offset/2)  )
        
        text (i+0.0+graph.offset, 0-2.0*graph.offset, "70%" , cex=1.2, adj=c(0.0, 0.5))
#        text (i+0.5             , 0-2.0*graph.offset, "85%", cex=1.2, adj=c(0.5, 0.5))
        text (i+1.0-graph.offset, 0-2.0*graph.offset, "100%" , cex=1.2, adj=c(1.0, 0.5))

        text (i+0.5             , ylimits[2]        , platforms[i+1], cex=2.0, adj=c(0.5, 1.0))
       }
      text (1.5, ylimits[1], "Imputation accuracy", cex=2.5, adj=c(0.5, 0.0)) 

 
      # y: Efficacy
      for (i in 0:2) {
        lines  (x=rep(0,2)              , y=i+  c(0.0+graph.offset,1.0-graph.offset)  )
          lines(x=0+c(0,-graph.offset/2), y=i+rep(0.0+graph.offset,2)                 )
          lines(x=0+c(0,-graph.offset/2), y=i+rep(0.5             ,2)                 )
          lines(x=0+c(0,-graph.offset/2), y=i+rep(1.0-graph.offset,2)                 )
        
        text (0-1.5*graph.offset, i+1.0-graph.offset, "100%", cex=1.2, adj=c(1.0, 1.0))
        text (0-1.5*graph.offset, i+0.5             ,  "50%", cex=1.2, adj=c(1.0, 0.5))
        text (0-1.5*graph.offset, i+0.0+graph.offset,   "0%", cex=1.2, adj=c(1.0, 0.0))

        text (1.0*xlimits[1] , i+0.5                , rev(platforms)[i+1]    , cex=2.0, srt=90, adj=c(0.5, 0.5)) 

       }
      text (0.5*xlimits[1], 1+0.5, "Imputation efficacy", cex=2.5, srt=90, adj=c(0.5, 0.5)) 


      # - Dotted lines - #
      for (i in 0:2) {
        for (j in 0:2) {
          if ((i+j)!=2) {
            # Horizontal
            lines ( x=i+c(0+graph.offset,1-graph.offset), y=rep(j+0.0+graph.offset,2), lty=1, col=8 )
            lines ( x=i+c(0+graph.offset,1-graph.offset), y=rep(j+1.0-graph.offset,2), lty=1, col=8 )
            for (k in 0.1*(1:9)) {
              lines ( x=i+c(0+graph.offset,1-graph.offset), 
                      y=rep(j+0.0+graph.offset  ,2)+ReScale(k,c(0.0,1.0),1.0-2*graph.offset), 
                      lty=3, col=8 )
             }

            # Vertical
            lines ( x=i+rep(0.0+graph.offset,2), y=j+  c(0.0+graph.offset  ,1.0-graph.offset), lty=1, col=8 )
            lines ( x=i+rep(1.0-graph.offset,2), y=j+  c(0.0+graph.offset  ,1.0-graph.offset), lty=1, col=8 )
            for (k in 0.1*(7:9)) {
              lines ( x=rep(i+graph.offset,2) + ReScale(k, c(0.7,1),1.0-2.0*graph.offset) ,
                      y=j+  c(0.0+graph.offset  , 1.0-graph.offset), 
                      lty=3, col=8 )
             }
           }
         }
       }
       

      # - Methods - #
      xy.origin <- c(2,0.5) ;  y.dist <- 0.1
      
      rect  ( xy.origin[1]+0.2, xy.origin[2]-2.5*y.dist, xy.origin[1]+0.85, xy.origin[2]+2.5*y.dist, border=8)
      lines ( xy.origin[1]+c(0.3,0.4), xy.origin[2]+1.5*rep(y.dist,2), lty=1, col=method.colors[4] )
      lines ( xy.origin[1]+c(0.3,0.4), xy.origin[2]+0.5*rep(y.dist,2), lty=1, col=method.colors[3] )
      lines ( xy.origin[1]+c(0.3,0.4), xy.origin[2]-0.5*rep(y.dist,2), lty=1, col=method.colors[2] )
      lines ( xy.origin[1]+c(0.3,0.4), xy.origin[2]-1.5*rep(y.dist,2), lty=1, col=method.colors[1] )

      text ( xy.origin[1]+0.5, xy.origin[2]+1.5*y.dist, label=method.order[4], cex=1.5, adj=c(0.0,0.5) )
      text ( xy.origin[1]+0.5, xy.origin[2]+0.5*y.dist, label=method.order[3], cex=1.5, adj=c(0.0,0.5) )
      text ( xy.origin[1]+0.5, xy.origin[2]-0.5*y.dist, label=method.order[2], cex=1.5, adj=c(0.0,0.5) )
      text ( xy.origin[1]+0.5, xy.origin[2]-1.5*y.dist, label=method.order[1], cex=1.5, adj=c(0.0,0.5) )


      # - Imputation results - #
      PlotCurves ("Affy5", "Affy6", xy.origin=c(1,2),    methods=method.order, colors=method.colors, x.default=method.defaults, graph.offset=graph.offset) 
      PlotCurves ("Affy5", "Illu" , xy.origin=c(2,2),    methods=method.order, colors=method.colors, x.default=method.defaults, graph.offset=graph.offset) 
     
      PlotCurves ("Affy6", "Affy5", xy.origin=c(0,1),    methods=method.order, colors=method.colors, x.default=method.defaults, graph.offset=graph.offset) 
      PlotCurves ("Affy6", "Illu" , xy.origin=c(2,1),    methods=method.order, colors=method.colors, x.default=method.defaults, graph.offset=graph.offset) 
     
      PlotCurves ("Illu" , "Affy5", xy.origin=c(0,0),    methods=method.order, colors=method.colors, x.default=method.defaults, graph.offset=graph.offset) 
      PlotCurves ("Illu" , "Affy6", xy.origin=c(1,0),    methods=method.order, colors=method.colors, x.default=method.defaults, graph.offset=graph.offset) 
     
      
   dev.off()


#################################################################################################
#################################################################################################
#################################################################################################

rm (list=ls())

# ------------------------------------------------------------ #
# --- Plot imputation performance separately for each      --- #
# ---   method, stratified by MAF
# ------------------------------------------------------------ #

# --------------------------------------- #
# - Color palettes                      - #
# --------------------------------------- #

Get.LD.Color <- function (value=NA, color.values=NA, color.steps=NA) {
  color <- color.values [length(color.values)]
  if (!is.na(value)) { color <-  ( color.values [ color.steps <= value ] )[1] }
  color
 }

LD.color.number    <- 5000
LD.color.sw.number <- 1000

# Color #
ld.blue2red.colors <- function (n) {
  if ((n <- as.integer(n[1])) > 0) { c( rainbow (n, start = 0, end = 4/6) ) }
    else                           { character(0)                           }
 }

# Black/white #
ld.white2black.colors <- function (n) {
  if ((n <- as.integer(n[1])) > 0) { gray ( (0:(n-1) ) / n) }
    else                           { character(0)                           }
 }


LD.colors        <- ld.blue2red.colors (LD.color.number)
LD.sw.colors     <- ld.white2black.colors (LD.color.sw.number)


#LD.colors       <- heat.colors    (LD.color.number)
#LD.colors       <- terrain.colors (LD.color.number)
#LD.colors       <- topo.colors    (LD.color.number)
#LD.colors       <- cm.colors      (LD.color.number)

LD.color.steps     <- 1 - 1/(LD.color.number   -1) * 0:(LD.color.number   -1)
LD.color.sw.steps  <- 1 - 1/(LD.color.sw.number-1) * 0:(LD.color.sw.number-1)

# --------------------------------------- #
# --------------------------------------- #



# --------------------------------------- #
# - Plotting functions                  - #
# --------------------------------------- #


  GetImputeAccuName <- function (array1=NA, array2=NA, method=NA, subdir="Concordance_Proportion_Data_allMAF") {
    midfix     <- "concordanceWith"
    suffix     <- "_MAFall"

    paste ( "./", subdir, "/", method, "/",  array1, midfix, array2, suffix, sep="")
  }
  
  
  GetImputePropName <- function (array1=NA, array2=NA, method=NA, subdir="Concordance_Proportion_Data_allMAF") {
    midfix     <- "Without"
    suffix     <- "_MAFall"

#    paste ( "./", subdir, "/", method, "/",  array1, midfix, array2, suffix, sep="")
    paste ( "./", subdir, "/", method, "/",  array2, midfix, array1, suffix, sep="")
  }
  
    GetImputeAccuName ("Affy5", "Affy6", "MACH")
    GetImputePropName ("Affy5", "Affy6", "MACH")
  

  GetAccuRate <- function (array1=NA, array2=NA, method=NA) {
    read.table ( GetImputeAccuName (array1, array2, method), as.is=T, header=T )
   }
  
  GetPropRate <- function (array1=NA, array2=NA, method=NA) {
    read.table ( GetImputePropName (array1, array2, method), as.is=T, header=T )
   }

   GetAccuRate ("Affy5", "Affy6", "MACH")
   GetPropRate ("Affy5", "Affy6", "MACH")
   
   GetAccuRate ("Affy5", "Affy6", "BEAGLE")
   GetPropRate ("Affy5", "Affy6", "BEAGLE")
   

   
  GetAccuRates <- function (array1=NA, array2=NA, method=NA) {
  
    a.accu <- GetAccuRate (array1, array2, method)

    maf.names <- c("001", "001-005", "005-01", "01-02", "02-03", "03-04", "04-05")

    accu.rates <- cbind ( Threshold=a.accu[ a.accu[,1]==maf.names[1] ,2] )
    for (maf in maf.names) {
      accu.rates <- cbind ( accu.rates , a.accu[ a.accu[,1]==maf ,3] )
     }
    colnames(accu.rates)[1+1:length(maf.names)] <- maf.names

    accu.rates
   }

  GetPropRates <- function (array1=NA, array2=NA, method=NA) {
  
    a.prop <- GetPropRate (array1, array2, method)

    maf.names <- c("001", "001-005", "005-01", "01-02", "02-03", "03-04", "04-05")

    prop.rates <- cbind ( Threshold=a.prop[ a.prop[,1]==maf.names[1] ,2] )
    for (maf in maf.names) {
      aa.prop <- a.prop[ a.prop[,1]==maf , ]
      prop.rates <- cbind ( prop.rates , aa.prop[,3]/aa.prop[,4] )
     }
    colnames(prop.rates)[1+1:length(maf.names)] <- maf.names

    prop.rates
   }

    GetAccuRates("Affy5", "Affy6", "MACH")
    GetPropRates("Affy5", "Affy6", "MACH")

    GetAccuRates("Affy5", "Affy6", "BEAGLE")
    GetPropRates("Affy5", "Affy6", "BEAGLE")


  ReScale <- function (values=NA, range=NA, size=NA) {
    (values-range[1])/(range[2]-range[1]) * size
   }


  # Accuray
  PlotAccuEffiCurve <- function (array1=NA, array2=NA, xy.origin=NA, 
                                 accu.range=c(0.7,1.0), prop.range=c(0.0,1.0), x.default=NA,
                                 method=NA, colors=1, graph.offset=NA) {

    accu.rates <- GetAccuRates(array1, array2, method)
    prop.rates <- GetPropRates(array1, array2, method)
    maf.names  <- colnames(accu.rates)[-1]

    th   <- accu.rates[,"Threshold"] ;  th   <- ReScale ( th  , c(0,1)     , 1.0-2.0*graph.offset )  # rescale to zoomed area
    accu <- accu.rates[,maf.names  ] ;  accu <- ReScale ( accu, accu.range , 1.0-2.0*graph.offset )  # rescale to zoomed area
    prop <- prop.rates[,maf.names  ] ;  prop <- ReScale ( prop, prop.range , 1.0-2.0*graph.offset )  # rescale to zoomed area

    for (i in 1:(dim(accu)[2])) {
    
      x.lines  <- accu[,i]  ;  y.lines  <- prop[,i]
      x.points <- accu[,i]  ;  y.points <- prop[,i]

      # Do not draw lines with both endpoints outside figure range
      for (j in (length(x.lines)):2) {
        # Both side outside plotting range
        if ( !any(is.na(x.lines[j:(j-1)])) ) {
          if ( (x.lines[j]<0                 ) & (x.lines[j-1]<0                 ) ) { x.lines[j] <- NA ;  y.lines[j] <- NA }  else  {
          if ( (x.lines[j]>1-2.0*graph.offset) & (x.lines[j-1]>1-2.0*graph.offset) ) { x.lines[j] <- NA ;  y.lines[j] <- NA }  else  {
          # One point inside plotting range
          if ( (x.lines[j]<0) & (x.lines[j-1]>0) ) {
            y.lines[j] <- y.lines[j-1] + ( (x.lines[j]-0) / (x.lines[j]-x.lines[j-1]) )*(y.lines[j]-y.lines[j-1])
            x.lines[j] <- 0  
           }                                                                                                                   else  {
          if ( (x.lines[j]>0) & (x.lines[j-1]<0) ) {
            y.lines[j-1] <- y.lines[j] + ( (x.lines[j-1]-0) / (x.lines[j-1]-x.lines[j]) )*(y.lines[j-1]-y.lines[j])
            x.lines[j-1] <- 0  
           }                                                                                                                    } } }
         }
       }
      # Do not draw points outside figure range
      for (j in 1:(length(x.points))) {
        if ( !is.na(x.points[j]) ) {
          if ( (x.points[j]<0) | (x.points[j]>1-2.0*graph.offset) ) { x.points[j] <- NA ;  y.points[j] <- NA }
         }
       }
    
      lines  ( xy.origin[1]+graph.offset+x.lines , xy.origin[2]+graph.offset+y.lines , lty=1 , col=colors[i] )
      points ( xy.origin[1]+graph.offset+x.points, xy.origin[2]+graph.offset+y.points, pch=20, col=colors[i] )
      th.default <- accu.rates[,"Threshold"] == x.default ;  th.default[is.na(th.default)] <- F
      if (any(th.default)) {
        points ( xy.origin[1]+graph.offset+x.points[th.default], xy.origin[2]+graph.offset+y.points[th.default], pch=23, col=colors[i], cex=1.5 )
       }
     }

   }


  PlotCurves <- function (array1=NA, array2=NA, xy.origin=NA, 
                          accu.range=c(0.7,1.0), prop.range=c(0.0,1.0), x.default=NA,
                          method=NA, colors=1, graph.offset=NA) {

    PlotAccuEffiCurve (array1=array1, array2=array2, xy.origin=xy.origin, x.default=x.default,
                       accu.range=accu.range, prop.range=prop.range, 
                       method=method, colors=colors, graph.offset=graph.offset) 
                   
   }
   
   
   
# --------------------------------------- #
# --------------------------------------- #





# --------------------------------------- #
# - Imputation plots                    - #
# --------------------------------------- #

  xlimits <- c(-0.6,3.0)
  ylimits <- c(-0.3,3.3)

  graph.offset <- 0.03
  
  platforms <- c("Affymetrix 5.0", "Affymetrix 6.0", "Illumina 550k")
  
  subdirname      <- "Concordance_Proportion_Data_allMAF"
  programs        <- c("PLINK", "MACH", "IMPUTE", "BEAGLE")
  method.defaults <- c(0.95, 0.30, 0.90, 0.4)
  
  maf.colors   <- c(1, 2:7)

  maf.legend.order    <- c("MAF", "0.01-0.05", "0.05-0.10", "0.10-0.20", "0.20-0.30", "0.30-0.40", "0.40-0.50", "All")
  maf.legend.colors   <- c(1, 2:7, 1)
  maf.legend.lty      <- c(0, rep(1, length(maf.legend.order)))


  
  # --- Imputation using CEU --- #
  

  graph.file1     <- "accuracy_vs_efficacy_byMAF_"
  graph.file2     <- ".pdf"
  graph.title    <- NA
  graph.subtitle <- NA
  
  
for (pp in 1:length(programs)) {

  program    <- programs[pp]
  ct.default <- method.defaults[pp]

  pdf        (file=paste(graph.file1, program, graph.file2, sep=""), 
              paper="special", height=4*(xlimits[2]-xlimits[1]), width=4*(xlimits[2]-xlimits[1]))
  
    plot (0,0, type="n", bty="n",
          xlim=xlimits, ylim=ylimits,
          xaxt="n", yaxt="n", xlab="", ylab="",
          main=graph.title  , cex.main=2.0,
          sub=graph.subtitle, cex.sub =1.3)

      # - Legends - #
      # x: Accuracy 
#      for (i in 0:2) {
#        lines  (x=i+c(0+graph.offset,1-graph.offset), y=rep(0,2)                )
#          lines(x=i+rep(0.0+graph.offset,2)           , y=0+c(0,-graph.offset/2)  )
#          lines(x=i+rep(0.5             ,2)           , y=0+c(0,-graph.offset/2)  )
#          lines(x=i+rep(1.0-graph.offset,2)           , y=0+c(0,-graph.offset/2)  )
#        
#        text (i+0.0+graph.offset, 0-2.0*graph.offset, "60%" , cex=1.2, adj=c(0.0, 0.5))
#        text (i+0.5             , 0-2.0*graph.offset, "80%", cex=1.2, adj=c(0.5, 0.5))
#        text (i+1.0-graph.offset, 0-2.0*graph.offset, "100%" , cex=1.2, adj=c(1.0, 0.5))
#
#        text (i+0.5             , ylimits[2]        , platforms[i+1], cex=2.0, adj=c(0.5, 1.0))
#       }
#      text (1.5, ylimits[1], "Imputation accuracy", cex=2.5, adj=c(0.5, 0.0)) 

      for (i in 0:2) {
        lines  (x=i+c(0+graph.offset,1-graph.offset), y=rep(0,2)                )
          lines(x=i+rep(0.0+graph.offset,2)           , y=0+c(0,-graph.offset/2)  )
#          lines(x=i+rep(0.5             ,2)           , y=0+c(0,-graph.offset/2)  )
          lines(x=i+rep(1.0-graph.offset,2)           , y=0+c(0,-graph.offset/2)  )
        
        text (i+0.0+graph.offset, 0-2.0*graph.offset, "70%" , cex=1.2, adj=c(0.0, 0.5))
#        text (i+0.5             , 0-2.0*graph.offset, "85%", cex=1.2, adj=c(0.5, 0.5))
        text (i+1.0-graph.offset, 0-2.0*graph.offset, "100%" , cex=1.2, adj=c(1.0, 0.5))

        text (i+0.5             , ylimits[2]        , platforms[i+1], cex=2.0, adj=c(0.5, 1.0))
       }
      text (1.5, ylimits[1], "Imputation accuracy", cex=2.5, adj=c(0.5, 0.0)) 

 
      # y: Efficacy
      for (i in 0:2) {
        lines  (x=rep(0,2)              , y=i+  c(0.0+graph.offset,1.0-graph.offset)  )
          lines(x=0+c(0,-graph.offset/2), y=i+rep(0.0+graph.offset,2)                 )
          lines(x=0+c(0,-graph.offset/2), y=i+rep(0.5             ,2)                 )
          lines(x=0+c(0,-graph.offset/2), y=i+rep(1.0-graph.offset,2)                 )
        
        text (0-1.5*graph.offset, i+1.0-graph.offset, "100%", cex=1.2, adj=c(1.0, 1.0))
        text (0-1.5*graph.offset, i+0.5             ,  "50%", cex=1.2, adj=c(1.0, 0.5))
        text (0-1.5*graph.offset, i+0.0+graph.offset,   "0%", cex=1.2, adj=c(1.0, 0.0))

        text (1.0*xlimits[1] , i+0.5                , rev(platforms)[i+1]    , cex=2.0, srt=90, adj=c(0.5, 0.5)) 

       }
      text (0.5*xlimits[1], 1+0.5, "Imputation efficacy", cex=2.5, srt=90, adj=c(0.5, 0.5)) 


      # - Dotted lines - #
      for (i in 0:2) {
        for (j in 0:2) {
          if ((i+j)!=2) {
            # Horizontal
            lines ( x=i+c(0+graph.offset,1-graph.offset), y=rep(j+0.0+graph.offset,2), lty=1, col=8 )
            lines ( x=i+c(0+graph.offset,1-graph.offset), y=rep(j+1.0-graph.offset,2), lty=1, col=8 )
            for (k in 0.1*(1:9)) {
              lines ( x=i+c(0+graph.offset,1-graph.offset), 
                      y=rep(j+0.0+graph.offset  ,2)+ReScale(k,c(0.0,1.0),1.0-2*graph.offset), 
                      lty=3, col=8 )
             }

            # Vertical
            lines ( x=i+rep(0.0+graph.offset,2), y=j+  c(0.0+graph.offset  ,1.0-graph.offset), lty=1, col=8 )
            lines ( x=i+rep(1.0-graph.offset,2), y=j+  c(0.0+graph.offset  ,1.0-graph.offset), lty=1, col=8 )
            for (k in 0.1*(7:9)) {
              lines ( x=rep(i+graph.offset,2) + ReScale(k, c(0.7,1),1.0-2.0*graph.offset) ,
                      y=j+  c(0.0+graph.offset  , 1.0-graph.offset), 
                      lty=3, col=8 )
             }

           }
         }
       }
       

      # - Method - #
      xy.origin <- c(0,2.5) ;  y.dist <- 0.1

#      rect ( xy.origin[1]+0.2, xy.origin[2]-2*y.dist, xy.origin[1]+0.85, xy.origin[2]+2*y.dist, border=8)
      text ( xy.origin[1]+0.5, xy.origin[2]       , label=program, cex=3.0, adj=c(0.5,0.5) )

      # - MAF - #
      xy.origin <- c(2,0.95) ;  y.dist <- 0.1

      rect  ( xy.origin[1]+0.15, xy.origin[2]-0.5*y.dist, xy.origin[1]+0.85, xy.origin[2]-(length(maf.legend.order)+0.5)*y.dist, border=8)
      for (i in 1:length(maf.legend.order)) {
        lines ( xy.origin[1]+c(0.25,0.35), xy.origin[2]-rep(i*y.dist,2), lty=maf.legend.lty[i], col=maf.legend.colors[i] )
       }
      for (i in 1:length(maf.legend.order)) {
        text ( xy.origin[1]+0.45, xy.origin[2]-i*y.dist, label=maf.legend.order[i], cex=1.5, adj=c(0.0,0.5) )
       }


      # - Imputation results - #
      PlotCurves ("Affy5", "Affy6"   , xy.origin=c(1,2),    method=program, colors=maf.colors, x.default=ct.default, graph.offset=graph.offset) 
      PlotCurves ("Affy5", "Illu550" , xy.origin=c(2,2),    method=program, colors=maf.colors, x.default=ct.default, graph.offset=graph.offset) 
     
      PlotCurves ("Affy6", "Affy5"   , xy.origin=c(0,1),    method=program, colors=maf.colors, x.default=ct.default, graph.offset=graph.offset) 
      PlotCurves ("Affy6", "Illu550" , xy.origin=c(2,1),    method=program, colors=maf.colors, x.default=ct.default, graph.offset=graph.offset) 
     
      PlotCurves ("Illu550" , "Affy5", xy.origin=c(0,0),    method=program, colors=maf.colors, x.default=ct.default, graph.offset=graph.offset) 
      PlotCurves ("Illu550" , "Affy6", xy.origin=c(1,0),    method=program, colors=maf.colors, x.default=ct.default, graph.offset=graph.offset) 
     
      
   dev.off()

 }

################################################################################
### END OF SCRIPT ##############################################################
################################################################################
